@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2010 The Music Player Daemon Project
+ * Copyright (C) 2003-2011 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -20,10 +20,16 @@
 #ifndef MPD_ARCHIVE_LIST_H
 #define MPD_ARCHIVE_LIST_H
 
-#include <stdio.h>
-
 struct archive_plugin;
 
+extern const struct archive_plugin *const archive_plugins[];
+
+#define archive_plugins_for_each(plugin) \
+	for (const struct archive_plugin *plugin, \
+		*const*archive_plugin_iterator = &archive_plugins[0]; \
+		(plugin = *archive_plugin_iterator) != NULL; \
+		++archive_plugin_iterator)
+
 /* interface for using plugins */
 
 const struct archive_plugin *
@@ -32,8 +38,6 @@ archive_plugin_from_suffix(const char *suffix);
 const struct archive_plugin *
 archive_plugin_from_name(const char *name);
 
-void archive_plugin_print_all_suffixes(FILE * fp);
-
 /* this is where we "load" all the "plugins" ;-) */
 void archive_plugin_init_all(void);
 
