@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2010 The Music Player Daemon Project
+ * Copyright (C) 2003-2011 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -20,6 +20,7 @@
 #ifndef MPD_AUDIO_FORMAT_H
 #define MPD_AUDIO_FORMAT_H
 
+#include <glib.h>
 #include <stdint.h>
 #include <stdbool.h>
 #include <assert.h>
@@ -30,11 +31,6 @@ enum sample_format {
 	SAMPLE_FORMAT_S8,
 	SAMPLE_FORMAT_S16,
 
-	/**
-	 * Signed 24 bit integer samples, without padding.
-	 */
-	SAMPLE_FORMAT_S24,
-
 	/**
 	 * Signed 24 bit integer samples, packed in 32 bit integers
 	 * (the most significant byte is filled with the sign bit).
@@ -42,8 +38,22 @@ enum sample_format {
 	SAMPLE_FORMAT_S24_P32,
 
 	SAMPLE_FORMAT_S32,
+
+	/**
+	 * 32 bit floating point samples in the host's format.  The
+	 * range is -1.0f to +1.0f.
+	 */
+	SAMPLE_FORMAT_FLOAT,
+
+	/**
+	 * Direct Stream Digital.  1-bit samples; each frame has one
+	 * byte (8 samples) per channel.
+	 */
+	SAMPLE_FORMAT_DSD,
 };
 
+static const unsigned MAX_CHANNELS = 8;
+
 /**
  * This structure describes the format of a raw PCM stream.
  */
@@ -66,13 +76,6 @@ struct audio_format {
 	 * fully supported currently.
 	 */
 	uint8_t channels;
-
-	/**
-	 * If zero, then samples are stored in host byte order.  If
-	 * nonzero, then samples are stored in the reverse host byte
-	 * order.
-	 */
-	uint8_t reverse_endian;
 };
 
 /**
@@ -91,7 +94,6 @@ static inline void audio_format_clear(struct audio_format *af)
 	af->sample_rate = 0;
 	af->format = SAMPLE_FORMAT_UNDEFINED;
 	af->channels = 0;
-	af->reverse_endian = 0;
 }
 
 /**
@@ -105,7 +107,6 @@ static inline void audio_format_init(struct audio_format *af,
 	af->sample_rate = sample_rate;
 	af->format = (uint8_t)format;
 	af->channels = channels;
-	af->reverse_endian = 0;
 }
 
 /**
@@ -162,9 +163,10 @@ audio_valid_sample_format(enum sample_format format)
 	switch (format) {
 	case SAMPLE_FORMAT_S8:
 	case SAMPLE_FORMAT_S16:
-	case SAMPLE_FORMAT_S24:
 	case SAMPLE_FORMAT_S24_P32:
 	case SAMPLE_FORMAT_S32:
+	case SAMPLE_FORMAT_FLOAT:
+	case SAMPLE_FORMAT_DSD:
 		return true;
 
 	case SAMPLE_FORMAT_UNDEFINED:
@@ -180,13 +182,14 @@ audio_valid_sample_format(enum sample_format format)
 static inline bool
 audio_valid_channel_count(unsigned channels)
 {
-	return channels >= 1 && channels <= 8;
+	return channels >= 1 && channels <= MAX_CHANNELS;
 }
 
 /**
  * Returns false if the format is not valid for playback with MPD.
  * This function performs some basic validity checks.
  */
+G_GNUC_PURE
 static inline bool audio_format_valid(const struct audio_format *af)
 {
 	return audio_valid_sample_rate(af->sample_rate) &&
@@ -198,6 +201,7 @@ static inline bool audio_format_valid(const struct audio_format *af)
  * Returns false if the format mask is not valid for playback with
  * MPD.  This function performs some basic validity checks.
  */
+G_GNUC_PURE
 static inline bool audio_format_mask_valid(const struct audio_format *af)
 {
 	return (af->sample_rate == 0 ||
@@ -212,58 +216,54 @@ static inline bool audio_format_equals(const struct audio_format *a,
 {
 	return a->sample_rate == b->sample_rate &&
 		a->format == b->format &&
-		a->channels == b->channels &&
-		a->reverse_endian == b->reverse_endian;
+		a->channels == b->channels;
 }
 
-static inline void
+void
 audio_format_mask_apply(struct audio_format *af,
-			const struct audio_format *mask)
+			const struct audio_format *mask);
+
+G_GNUC_CONST
+static inline unsigned
+sample_format_size(enum sample_format format)
 {
-	assert(audio_format_valid(af));
-	assert(audio_format_mask_valid(mask));
-
-	if (mask->sample_rate != 0)
-		af->sample_rate = mask->sample_rate;
-
-	if (mask->format != SAMPLE_FORMAT_UNDEFINED)
-		af->format = mask->format;
-
-	if (mask->channels != 0)
-		af->channels = mask->channels;
-
-	assert(audio_format_valid(af));
-}
-
-/**
- * Returns the size of each (mono) sample in bytes.
- */
-static inline unsigned audio_format_sample_size(const struct audio_format *af)
-{
-	switch (af->format) {
+	switch (format) {
 	case SAMPLE_FORMAT_S8:
 		return 1;
 
 	case SAMPLE_FORMAT_S16:
 		return 2;
 
-	case SAMPLE_FORMAT_S24:
-		return 3;
-
 	case SAMPLE_FORMAT_S24_P32:
 	case SAMPLE_FORMAT_S32:
+	case SAMPLE_FORMAT_FLOAT:
 		return 4;
 
+	case SAMPLE_FORMAT_DSD:
+		/* each frame has 8 samples per channel */
+		return 1;
+
 	case SAMPLE_FORMAT_UNDEFINED:
-		break;
+		return 0;
 	}
 
+	assert(false);
 	return 0;
 }
 
+/**
+ * Returns the size of each (mono) sample in bytes.
+ */
+G_GNUC_PURE
+static inline unsigned audio_format_sample_size(const struct audio_format *af)
+{
+	return sample_format_size((enum sample_format)af->format);
+}
+
 /**
  * Returns the size of each full frame in bytes.
  */
+G_GNUC_PURE
 static inline unsigned
 audio_format_frame_size(const struct audio_format *af)
 {
@@ -274,6 +274,7 @@ audio_format_frame_size(const struct audio_format *af)
  * Returns the floating point factor which converts a time span to a
  * storage size in bytes.
  */
+G_GNUC_PURE
 static inline double audio_format_time_to_size(const struct audio_format *af)
 {
 	return af->sample_rate * audio_format_frame_size(af);
@@ -286,6 +287,7 @@ static inline double audio_format_time_to_size(const struct audio_format *af)
  * @param format a #sample_format enum value
  * @return the string
  */
+G_GNUC_PURE G_GNUC_MALLOC
 const char *
 sample_format_to_string(enum sample_format format);
 
@@ -297,6 +299,7 @@ sample_format_to_string(enum sample_format format);
  * @param s a buffer to print into
  * @return the string, or NULL if the #audio_format object is invalid
  */
+G_GNUC_PURE G_GNUC_MALLOC
 const char *
 audio_format_to_string(const struct audio_format *af,
 		       struct audio_format_string *s);
