@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2010 The Music Player Daemon Project
+ * Copyright (C) 2003-2011 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -26,6 +26,7 @@
 #include "audio_parser.h"
 #include "audio_format.h"
 #include "audio_check.h"
+#include "gcc.h"
 
 #include <assert.h>
 #include <string.h>
@@ -81,6 +82,18 @@ parse_sample_format(const char *src, bool mask,
 		return true;
 	}
 
+	if (*src == 'f') {
+		*sample_format_r = SAMPLE_FORMAT_FLOAT;
+		*endptr_r = src + 1;
+		return true;
+	}
+
+	if (memcmp(src, "dsd", 3) == 0) {
+		*sample_format_r = SAMPLE_FORMAT_DSD;
+		*endptr_r = src + 3;
+		return true;
+	}
+
 	value = strtoul(src, &endptr, 10);
 	if (endptr == src) {
 		g_set_error(error_r, audio_parser_quark(), 0,
@@ -98,11 +111,11 @@ parse_sample_format(const char *src, bool mask,
 		break;
 
 	case 24:
-		if (memcmp(endptr, "_3", 2) == 0) {
-			sample_format = SAMPLE_FORMAT_S24;
+		if (memcmp(endptr, "_3", 2) == 0)
+			/* for backwards compatibility */
 			endptr += 2;
-		} else
-			sample_format = SAMPLE_FORMAT_S24_P32;
+
+		sample_format = SAMPLE_FORMAT_S24_P32;
 		break;
 
 	case 32:
@@ -160,6 +173,11 @@ audio_format_parse(struct audio_format *dest, const char *src,
 
 	/* parse sample rate */
 
+#if GCC_CHECK_VERSION(4,7)
+	/* workaround -Wmaybe-uninitialized false positive */
+	rate = 0;
+#endif
+
 	if (!parse_sample_rate(src, mask, &rate, &src, error_r))
 		return false;
 
@@ -171,6 +189,11 @@ audio_format_parse(struct audio_format *dest, const char *src,
 
 	/* parse sample format */
 
+#if GCC_CHECK_VERSION(4,7)
+	/* workaround -Wmaybe-uninitialized false positive */
+	sample_format = SAMPLE_FORMAT_UNDEFINED;
+#endif
+
 	if (!parse_sample_format(src, mask, &sample_format, &src, error_r))
 		return false;
 
