@@ -1579,7 +1579,7 @@ handle_playlistadd(struct client *client, G_GNUC_UNUSED int argc, char *argv[])
 			return COMMAND_RETURN_ERROR;
 		}
 
-		success = spl_append_uri(argv[1], playlist, &error);
+		success = spl_append_uri(uri, playlist, &error);
 	} else
 		success = addAllInToStoredPlaylist(uri, playlist, &error);
 
