@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2010 The Music Player Daemon Project
+ * Copyright (C) 2003-2011 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -124,11 +124,10 @@ void flac_metadata_common_cb(const FLAC__StreamMetadata * block,
 	case FLAC__METADATA_TYPE_VORBIS_COMMENT:
 		if (flac_parse_replay_gain(&rgi, block))
 			replay_gain_db = decoder_replay_gain(data->decoder, &rgi);
-		if (flac_parse_mixramp(&mixramp_start, &mixramp_end, block)) {
-			g_debug("setting mixramp_tags");
+
+		if (flac_parse_mixramp(&mixramp_start, &mixramp_end, block))
 			decoder_mixramp(data->decoder, replay_gain_db,
 					mixramp_start, mixramp_end);
-		}
 
 		if (data->tag != NULL)
 			flac_vorbis_comments_to_tag(data->tag, NULL,
@@ -139,26 +138,13 @@ void flac_metadata_common_cb(const FLAC__StreamMetadata * block,
 	}
 }
 
-void flac_error_common_cb(const char *plugin,
-			  const FLAC__StreamDecoderErrorStatus status,
+void flac_error_common_cb(const FLAC__StreamDecoderErrorStatus status,
 			  struct flac_data *data)
 {
 	if (decoder_get_command(data->decoder) == DECODE_COMMAND_STOP)
 		return;
 
-	switch (status) {
-	case FLAC__STREAM_DECODER_ERROR_STATUS_LOST_SYNC:
-		g_warning("%s lost sync\n", plugin);
-		break;
-	case FLAC__STREAM_DECODER_ERROR_STATUS_BAD_HEADER:
-		g_warning("bad %s header\n", plugin);
-		break;
-	case FLAC__STREAM_DECODER_ERROR_STATUS_FRAME_CRC_MISMATCH:
-		g_warning("%s crc mismatch\n", plugin);
-		break;
-	default:
-		g_warning("unknown %s error\n", plugin);
-	}
+	g_warning("%s", FLAC__StreamDecoderErrorStatusString[status]);
 }
 
 /**
