@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2010 The Music Player Daemon Project
+ * Copyright (C) 2003-2011 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -58,6 +58,12 @@ struct decoder_control {
 	 */
 	GCond *cond;
 
+	/**
+	 * The trigger of this object's client.  It is signalled
+	 * whenever an event occurs.
+	 */
+	GCond *client_cond;
+
 	enum decoder_state state;
 	enum decoder_command command;
 
@@ -114,11 +120,12 @@ struct decoder_control {
 	char *mixramp_prev_end;
 };
 
-void
-dc_init(struct decoder_control *dc);
+G_GNUC_MALLOC
+struct decoder_control *
+dc_new(GCond *client_cond);
 
 void
-dc_deinit(struct decoder_control *dc);
+dc_free(struct decoder_control *dc);
 
 /**
  * Locks the #decoder_control object.
@@ -234,9 +241,6 @@ decoder_current_song(const struct decoder_control *dc)
 	return NULL;
 }
 
-void
-dc_command_wait(struct decoder_control *dc);
-
 /**
  * Start the decoder.
  *
