@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2010 The Music Player Daemon Project
+ * Copyright (C) 2003-2011 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -20,7 +20,6 @@
 #include "config.h"
 #include "decoder_internal.h"
 #include "decoder_control.h"
-#include "player_control.h"
 #include "pipe.h"
 #include "input_stream.h"
 #include "buffer.h"
@@ -28,44 +27,20 @@
 
 #include <assert.h>
 
-/**
- * This is a wrapper for input_stream_buffer().  It assumes that the
- * decoder is currently locked, and temporarily unlocks it while
- * calling input_stream_buffer().  We shouldn't hold the lock during a
- * potentially blocking operation.
- */
-static bool
-decoder_input_buffer(struct decoder_control *dc, struct input_stream *is)
-{
-	GError *error = NULL;
-	int ret;
-
-	decoder_unlock(dc);
-	ret = input_stream_buffer(is, &error);
-	if (ret < 0) {
-		g_warning("%s", error->message);
-		g_error_free(error);
-	}
-
-	decoder_lock(dc);
-
-	return ret > 0;
-}
-
 /**
  * All chunks are full of decoded data; wait for the player to free
  * one.
  */
 static enum decoder_command
-need_chunks(struct decoder_control *dc, struct input_stream *is, bool do_wait)
+need_chunks(struct decoder_control *dc, bool do_wait)
 {
 	if (dc->command == DECODE_COMMAND_STOP ||
 	    dc->command == DECODE_COMMAND_SEEK)
 		return dc->command;
 
-	if ((is == NULL || !decoder_input_buffer(dc, is)) && do_wait) {
+	if (do_wait) {
 		decoder_wait(dc);
-		player_signal();
+		g_cond_signal(dc->client_cond);
 
 		return dc->command;
 	}
@@ -74,7 +49,7 @@ need_chunks(struct decoder_control *dc, struct input_stream *is, bool do_wait)
 }
 
 struct music_chunk *
-decoder_get_chunk(struct decoder *decoder, struct input_stream *is)
+decoder_get_chunk(struct decoder *decoder)
 {
 	struct decoder_control *dc = decoder->dc;
 	enum decoder_command cmd;
@@ -97,7 +72,7 @@ decoder_get_chunk(struct decoder *decoder, struct input_stream *is)
 		}
 
 		decoder_lock(dc);
-		cmd = need_chunks(dc, is, true);
+		cmd = need_chunks(dc, true);
 		decoder_unlock(dc);
 	} while (cmd == DECODE_COMMAND_NONE);
 
