@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2010 The Music Player Daemon Project
+ * Copyright (C) 2003-2011 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -22,11 +22,15 @@
 #include "server_socket.h"
 #include "client.h"
 #include "conf.h"
-#include "glib_compat.h"
+#include "main.h"
 
 #include <string.h>
 #include <assert.h>
 
+#ifdef ENABLE_SYSTEMD_DAEMON
+#include <systemd/sd-daemon.h>
+#endif
+
 #undef G_LOG_DOMAIN
 #define G_LOG_DOMAIN "listen"
 
@@ -39,7 +43,7 @@ static void
 listen_callback(int fd, const struct sockaddr *address,
 		size_t address_length, int uid, G_GNUC_UNUSED void *ctx)
 {
-	client_new(fd, address, address_length, uid);
+	client_new(global_player_control, fd, address, address_length, uid);
 }
 
 static bool
@@ -60,6 +64,30 @@ listen_add_config_param(unsigned int port,
 	}
 }
 
+static bool
+listen_systemd_activation(GError **error_r)
+{
+#ifdef ENABLE_SYSTEMD_DAEMON
+	int n = sd_listen_fds(true);
+	if (n <= 0) {
+		if (n < 0)
+			g_warning("sd_listen_fds() failed: %s",
+				  g_strerror(-n));
+		return false;
+	}
+
+	for (int i = SD_LISTEN_FDS_START, end = SD_LISTEN_FDS_START + n;
+	     i != end; ++i)
+		if (!server_socket_add_fd(listen_socket, i, error_r))
+			return false;
+
+	return true;
+#else
+	(void)error_r;
+	return false;
+#endif
+}
+
 bool
 listen_global_init(GError **error_r)
 {
@@ -71,6 +99,14 @@ listen_global_init(GError **error_r)
 
 	listen_socket = server_socket_new(listen_callback, NULL);
 
+	if (listen_systemd_activation(&error))
+		return true;
+
+	if (error != NULL) {
+		g_propagate_error(error_r, error);
+		return false;
+	}
+
 	if (param != NULL) {
 		/* "bind_to_address" is configured, create listeners
 		   for all values */
