@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2010 The Music Player Daemon Project
+ * Copyright (C) 2003-2011 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -23,6 +23,13 @@
 #include <glib.h>
 #include <stdbool.h>
 
+G_GNUC_CONST
+static inline GQuark
+log_quark(void)
+{
+	return g_quark_from_static_string("log");
+}
+
 /**
  * Configure a logging destination for daemon startup, before the
  * configuration file is read.  This allows the daemon to use the
@@ -34,12 +41,14 @@
 void
 log_early_init(bool verbose);
 
-void log_init(bool verbose, bool use_stdout);
+bool
+log_init(bool verbose, bool use_stdout, GError **error_r);
+
+void
+log_deinit(void);
 
 void setup_log_output(bool use_stdout);
 
 int cycle_log_files(void);
 
-void close_log_files(void);
-
 #endif /* LOG_H */
