@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2010 The Music Player Daemon Project
+ * Copyright (C) 2003-2011 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -18,12 +18,13 @@
  */
 
 #include "config.h"
+#include "httpd_output_plugin.h"
 #include "httpd_internal.h"
 #include "httpd_client.h"
 #include "output_api.h"
 #include "encoder_plugin.h"
 #include "encoder_list.h"
-#include "socket_util.h"
+#include "resolver.h"
 #include "page.h"
 #include "icy_server.h"
 #include "fd_util.h"
@@ -52,6 +53,31 @@ httpd_output_quark(void)
 	return g_quark_from_static_string("httpd_output");
 }
 
+/**
+ * Check whether there is at least one client.
+ *
+ * Caller must lock the mutex.
+ */
+G_GNUC_PURE
+static bool
+httpd_output_has_clients(const struct httpd_output *httpd)
+{
+	return httpd->clients != NULL;
+}
+
+/**
+ * Check whether there is at least one client.
+ */
+G_GNUC_PURE
+static bool
+httpd_output_lock_has_clients(const struct httpd_output *httpd)
+{
+	g_mutex_lock(httpd->mutex);
+	bool result = httpd_output_has_clients(httpd);
+	g_mutex_unlock(httpd->mutex);
+	return result;
+}
+
 static void
 httpd_listen_in_event(int fd, const struct sockaddr *address,
 		      size_t address_length, int uid, void *ctx);
@@ -78,15 +104,15 @@ httpd_output_unbind(struct httpd_output *httpd)
 	g_mutex_unlock(httpd->mutex);
 }
 
-static void *
-httpd_output_init(G_GNUC_UNUSED const struct audio_format *audio_format,
-		  const struct config_param *param,
+static struct audio_output *
+httpd_output_init(const struct config_param *param,
 		  GError **error)
 {
 	struct httpd_output *httpd = g_new(struct httpd_output, 1);
-	const char *encoder_name, *bind_to_address;
-	const struct encoder_plugin *encoder_plugin;
-	guint port;
+	if (!ao_base_init(&httpd->base, &httpd_output_plugin, param, error)) {
+		g_free(httpd);
+		return NULL;
+	}
 
 	/* read configuration */
 	httpd->name =
@@ -96,13 +122,17 @@ httpd_output_init(G_GNUC_UNUSED const struct audio_format *audio_format,
 	httpd->website =
 		config_get_block_string(param, "website", "Set website in config");
 
-	port = config_get_block_unsigned(param, "port", 8000);
+	guint port = config_get_block_unsigned(param, "port", 8000);
 
-	encoder_name = config_get_block_string(param, "encoder", "vorbis");
-	encoder_plugin = encoder_plugin_get(encoder_name);
+	const char *encoder_name =
+		config_get_block_string(param, "encoder", "vorbis");
+	const struct encoder_plugin *encoder_plugin =
+		encoder_plugin_get(encoder_name);
 	if (encoder_plugin == NULL) {
 		g_set_error(error, httpd_output_quark(), 0,
 			    "No such encoder: %s", encoder_name);
+		ao_base_finish(&httpd->base);
+		g_free(httpd);
 		return NULL;
 	}
 
@@ -112,15 +142,18 @@ httpd_output_init(G_GNUC_UNUSED const struct audio_format *audio_format,
 
 	httpd->server_socket = server_socket_new(httpd_listen_in_event, httpd);
 
-	bind_to_address =
+	const char *bind_to_address =
 		config_get_block_string(param, "bind_to_address", NULL);
 	bool success = bind_to_address != NULL &&
 		strcmp(bind_to_address, "any") != 0
 		? server_socket_add_host(httpd->server_socket, bind_to_address,
 					 port, error)
 		: server_socket_add_port(httpd->server_socket, port, error);
-	if (!success)
+	if (!success) {
+		ao_base_finish(&httpd->base);
+		g_free(httpd);
 		return NULL;
+	}
 
 	/* initialize metadata */
 	httpd->metadata = NULL;
@@ -129,8 +162,11 @@ httpd_output_init(G_GNUC_UNUSED const struct audio_format *audio_format,
 	/* initialize encoder */
 
 	httpd->encoder = encoder_init(encoder_plugin, param, error);
-	if (httpd->encoder == NULL)
+	if (httpd->encoder == NULL) {
+		ao_base_finish(&httpd->base);
+		g_free(httpd);
 		return NULL;
+	}
 
 	/* determine content type */
 	httpd->content_type = encoder_get_mime_type(httpd->encoder);
@@ -140,13 +176,13 @@ httpd_output_init(G_GNUC_UNUSED const struct audio_format *audio_format,
 
 	httpd->mutex = g_mutex_new();
 
-	return httpd;
+	return &httpd->base;
 }
 
 static void
-httpd_output_finish(void *data)
+httpd_output_finish(struct audio_output *ao)
 {
-	struct httpd_output *httpd = data;
+	struct httpd_output *httpd = (struct httpd_output *)ao;
 
 	if (httpd->metadata)
 		page_unref(httpd->metadata);
@@ -154,6 +190,7 @@ httpd_output_finish(void *data)
 	encoder_finish(httpd->encoder);
 	server_socket_free(httpd->server_socket);
 	g_mutex_free(httpd->mutex);
+	ao_base_finish(&httpd->base);
 	g_free(httpd);
 }
 
@@ -200,7 +237,7 @@ httpd_listen_in_event(int fd, const struct sockaddr *address,
 			g_warning("libwrap refused connection (libwrap=%s) from %s",
 			      progname, hostaddr);
 			g_free(hostaddr);
-			close(fd);
+			close_socket(fd);
 			g_mutex_unlock(httpd->mutex);
 			return;
 		}
@@ -221,7 +258,7 @@ httpd_listen_in_event(int fd, const struct sockaddr *address,
 		     httpd->clients_cnt < httpd->clients_max))
 			httpd_client_add(httpd, fd);
 		else
-			close(fd);
+			close_socket(fd);
 	} else if (fd < 0 && errno != EINTR) {
 		g_warning("accept() failed: %s", g_strerror(errno));
 	}
@@ -236,8 +273,6 @@ httpd_listen_in_event(int fd, const struct sockaddr *address,
 static struct page *
 httpd_output_read_page(struct httpd_output *httpd)
 {
-	size_t size = 0, nbytes;
-
 	if (httpd->unflushed_input >= 65536) {
 		/* we have fed a lot of input into the encoder, but it
 		   didn't give anything back yet - flush now to avoid
@@ -246,9 +281,11 @@ httpd_output_read_page(struct httpd_output *httpd)
 		httpd->unflushed_input = 0;
 	}
 
+	size_t size = 0;
 	do {
-		nbytes = encoder_read(httpd->encoder, httpd->buffer + size,
-				      sizeof(httpd->buffer) - size);
+		size_t nbytes = encoder_read(httpd->encoder,
+					     httpd->buffer + size,
+					     sizeof(httpd->buffer) - size);
 		if (nbytes == 0)
 			break;
 
@@ -268,10 +305,7 @@ httpd_output_encoder_open(struct httpd_output *httpd,
 			  struct audio_format *audio_format,
 			  GError **error)
 {
-	bool success;
-
-	success = encoder_open(httpd->encoder, audio_format, error);
-	if (!success)
+	if (!encoder_open(httpd->encoder, audio_format, error))
 		return false;
 
 	/* we have to remember the encoder header, i.e. the first
@@ -285,34 +319,32 @@ httpd_output_encoder_open(struct httpd_output *httpd,
 }
 
 static bool
-httpd_output_enable(void *data, GError **error_r)
+httpd_output_enable(struct audio_output *ao, GError **error_r)
 {
-	struct httpd_output *httpd = data;
+	struct httpd_output *httpd = (struct httpd_output *)ao;
 
 	return httpd_output_bind(httpd, error_r);
 }
 
 static void
-httpd_output_disable(void *data)
+httpd_output_disable(struct audio_output *ao)
 {
-	struct httpd_output *httpd = data;
+	struct httpd_output *httpd = (struct httpd_output *)ao;
 
 	httpd_output_unbind(httpd);
 }
 
 static bool
-httpd_output_open(void *data, struct audio_format *audio_format,
+httpd_output_open(struct audio_output *ao, struct audio_format *audio_format,
 		  GError **error)
 {
-	struct httpd_output *httpd = data;
-	bool success;
+	struct httpd_output *httpd = (struct httpd_output *)ao;
 
 	g_mutex_lock(httpd->mutex);
 
 	/* open the encoder */
 
-	success = httpd_output_encoder_open(httpd, audio_format, error);
-	if (!success) {
+	if (!httpd_output_encoder_open(httpd, audio_format, error)) {
 		g_mutex_unlock(httpd->mutex);
 		return false;
 	}
@@ -337,9 +369,10 @@ httpd_client_delete(gpointer data, G_GNUC_UNUSED gpointer user_data)
 	httpd_client_free(client);
 }
 
-static void httpd_output_close(void *data)
+static void
+httpd_output_close(struct audio_output *ao)
 {
-	struct httpd_output *httpd = data;
+	struct httpd_output *httpd = (struct httpd_output *)ao;
 
 	g_mutex_lock(httpd->mutex);
 
@@ -378,9 +411,22 @@ httpd_output_send_header(struct httpd_output *httpd,
 }
 
 static unsigned
-httpd_output_delay(void *data)
+httpd_output_delay(struct audio_output *ao)
 {
-	struct httpd_output *httpd = data;
+	struct httpd_output *httpd = (struct httpd_output *)ao;
+
+	if (!httpd_output_lock_has_clients(httpd) && httpd->base.pause) {
+		/* if there's no client and this output is paused,
+		   then httpd_output_pause() will not do anything, it
+		   will not fill the buffer and it will not update the
+		   timer; therefore, we reset the timer here */
+		timer_reset(httpd->timer);
+
+		/* some arbitrary delay that is long enough to avoid
+		   consuming too much CPU, and short enough to notice
+		   new clients quickly enough */
+		return 1000;
+	}
 
 	return httpd->timer->started
 		? timer_delay(httpd->timer)
@@ -442,10 +488,7 @@ static bool
 httpd_output_encode_and_play(struct httpd_output *httpd,
 			     const void *chunk, size_t size, GError **error)
 {
-	bool success;
-
-	success = encoder_write(httpd->encoder, chunk, size, error);
-	if (!success)
+	if (!encoder_write(httpd->encoder, chunk, size, error))
 		return false;
 
 	httpd->unflushed_input += size;
@@ -456,21 +499,13 @@ httpd_output_encode_and_play(struct httpd_output *httpd,
 }
 
 static size_t
-httpd_output_play(void *data, const void *chunk, size_t size, GError **error)
+httpd_output_play(struct audio_output *ao, const void *chunk, size_t size,
+		  GError **error_r)
 {
-	struct httpd_output *httpd = data;
-	bool has_clients;
+	struct httpd_output *httpd = (struct httpd_output *)ao;
 
-	g_mutex_lock(httpd->mutex);
-	has_clients = httpd->clients != NULL;
-	g_mutex_unlock(httpd->mutex);
-
-	if (has_clients) {
-		bool success;
-
-		success = httpd_output_encode_and_play(httpd, chunk, size,
-						       error);
-		if (!success)
+	if (httpd_output_lock_has_clients(httpd)) {
+		if (!httpd_output_encode_and_play(httpd, chunk, size, error_r))
 			return 0;
 	}
 
@@ -482,19 +517,15 @@ httpd_output_play(void *data, const void *chunk, size_t size, GError **error)
 }
 
 static bool
-httpd_output_pause(void *data)
+httpd_output_pause(struct audio_output *ao)
 {
-	struct httpd_output *httpd = data;
+	struct httpd_output *httpd = (struct httpd_output *)ao;
 
-	g_mutex_lock(httpd->mutex);
-	bool has_clients = httpd->clients != NULL;
-	g_mutex_unlock(httpd->mutex);
-
-	if (has_clients) {
+	if (httpd_output_lock_has_clients(httpd)) {
 		static const char silence[1020];
-		return httpd_output_play(data, silence, sizeof(silence), NULL);
+		return httpd_output_play(ao, silence, sizeof(silence),
+					 NULL) > 0;
 	} else {
-		g_usleep(100000);
 		return true;
 	}
 }
@@ -509,19 +540,18 @@ httpd_send_metadata(gpointer data, gpointer user_data)
 }
 
 static void
-httpd_output_tag(void *data, const struct tag *tag)
+httpd_output_tag(struct audio_output *ao, const struct tag *tag)
 {
-	struct httpd_output *httpd = data;
+	struct httpd_output *httpd = (struct httpd_output *)ao;
 
 	assert(tag != NULL);
 
 	if (httpd->encoder->plugin->tag != NULL) {
 		/* embed encoder tags */
-		struct page *page;
 
 		/* flush the current stream, and end it */
 
-		encoder_flush(httpd->encoder, NULL);
+		encoder_pre_tag(httpd->encoder, NULL);
 		httpd_output_encoder_to_clients(httpd);
 
 		/* send the tag to the encoder - which starts a new
@@ -533,7 +563,7 @@ httpd_output_tag(void *data, const struct tag *tag)
 		   used as the new "header" page, which is sent to all
 		   new clients */
 
-		page = httpd_output_read_page(httpd);
+		struct page *page = httpd_output_read_page(httpd);
 		if (page != NULL) {
 			if (httpd->header != NULL)
 				page_unref(httpd->header);
@@ -568,9 +598,9 @@ httpd_client_cancel_callback(gpointer data, G_GNUC_UNUSED gpointer user_data)
 }
 
 static void
-httpd_output_cancel(void *data)
+httpd_output_cancel(struct audio_output *ao)
 {
-	struct httpd_output *httpd = data;
+	struct httpd_output *httpd = (struct httpd_output *)ao;
 
 	g_mutex_lock(httpd->mutex);
 	g_list_foreach(httpd->clients, httpd_client_cancel_callback, NULL);
