@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2010 The Music Player Daemon Project
+ * Copyright (C) 2003-2011 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -18,6 +18,7 @@
  */
 
 #include "config.h"
+#include "jack_output_plugin.h"
 #include "output_api.h"
 
 #include <assert.h>
@@ -43,6 +44,8 @@ enum {
 static const size_t jack_sample_size = sizeof(jack_default_audio_sample_t);
 
 struct jack_data {
+	struct audio_output base;
+
 	/**
 	 * libjack options passed to jack_client_open().
 	 */
@@ -143,6 +146,13 @@ mpd_jack_process(jack_nframes_t nframes, void *arg)
 
 	for (unsigned i = 0; i < jd->audio_format.channels; ++i) {
 		out = jack_port_get_buffer(jd->ports[i], nframes);
+		if (out == NULL)
+			/* workaround for libjack1 bug: if the server
+			   connection fails, the process callback is
+			   invoked anyway, but unable to get a
+			   buffer */
+			continue;
+
 		jack_ringbuffer_read(jd->ringbuffer[i],
 				     (char *)out, available * jack_sample_size);
 
@@ -156,6 +166,12 @@ mpd_jack_process(jack_nframes_t nframes, void *arg)
 	for (unsigned i = jd->audio_format.channels;
 	     i < jd->num_source_ports; ++i) {
 		out = jack_port_get_buffer(jd->ports[i], nframes);
+		if (out == NULL)
+			/* workaround for libjack1 bug: if the server
+			   connection fails, the process callback is
+			   invoked anyway, but unable to get a
+			   buffer */
+			continue;
 
 		for (jack_nframes_t f = 0; f < nframes; ++f)
 			out[f] = 0.0;
@@ -291,14 +307,18 @@ parse_port_list(int line, const char *source, char **dest, GError **error_r)
 	return n;
 }
 
-static void *
-mpd_jack_init(G_GNUC_UNUSED const struct audio_format *audio_format,
-	      const struct config_param *param, GError **error_r)
+static struct audio_output *
+mpd_jack_init(const struct config_param *param, GError **error_r)
 {
-	struct jack_data *jd;
+	struct jack_data *jd = g_new(struct jack_data, 1);
+
+	if (!ao_base_init(&jd->base, &jack_output_plugin, param, error_r)) {
+		g_free(jd);
+		return NULL;
+	}
+
 	const char *value;
 
-	jd = g_new(struct jack_data, 1);
 	jd->options = JackNullOption;
 
 	jd->name = config_get_block_string(param, "client_name", NULL);
@@ -361,13 +381,13 @@ mpd_jack_init(G_GNUC_UNUSED const struct audio_format *audio_format,
 	jack_set_info_function(mpd_jack_info);
 #endif
 
-	return jd;
+	return &jd->base;
 }
 
 static void
-mpd_jack_finish(void *data)
+mpd_jack_finish(struct audio_output *ao)
 {
-	struct jack_data *jd = data;
+	struct jack_data *jd = (struct jack_data *)ao;
 
 	for (unsigned i = 0; i < jd->num_source_ports; ++i)
 		g_free(jd->source_ports[i]);
@@ -375,13 +395,14 @@ mpd_jack_finish(void *data)
 	for (unsigned i = 0; i < jd->num_destination_ports; ++i)
 		g_free(jd->destination_ports[i]);
 
+	ao_base_finish(&jd->base);
 	g_free(jd);
 }
 
 static bool
-mpd_jack_enable(void *data, GError **error_r)
+mpd_jack_enable(struct audio_output *ao, GError **error_r)
 {
-	struct jack_data *jd = (struct jack_data *)data;
+	struct jack_data *jd = (struct jack_data *)ao;
 
 	for (unsigned i = 0; i < jd->num_source_ports; ++i)
 		jd->ringbuffer[i] = NULL;
@@ -390,9 +411,9 @@ mpd_jack_enable(void *data, GError **error_r)
 }
 
 static void
-mpd_jack_disable(void *data)
+mpd_jack_disable(struct audio_output *ao)
 {
-	struct jack_data *jd = (struct jack_data *)data;
+	struct jack_data *jd = (struct jack_data *)ao;
 
 	if (jd->client != NULL)
 		mpd_jack_disconnect(jd);
@@ -555,14 +576,18 @@ mpd_jack_start(struct jack_data *jd, GError **error_r)
 }
 
 static bool
-mpd_jack_open(void *data, struct audio_format *audio_format, GError **error_r)
+mpd_jack_open(struct audio_output *ao, struct audio_format *audio_format,
+	      GError **error_r)
 {
-	struct jack_data *jd = data;
+	struct jack_data *jd = (struct jack_data *)ao;
 
 	assert(jd != NULL);
 
 	jd->pause = false;
 
+	if (jd->client != NULL && jd->shutdown)
+		mpd_jack_disconnect(jd);
+
 	if (jd->client == NULL && !mpd_jack_connect(jd, error_r))
 		return false;
 
@@ -576,13 +601,23 @@ mpd_jack_open(void *data, struct audio_format *audio_format, GError **error_r)
 }
 
 static void
-mpd_jack_close(G_GNUC_UNUSED void *data)
+mpd_jack_close(G_GNUC_UNUSED struct audio_output *ao)
 {
-	struct jack_data *jd = data;
+	struct jack_data *jd = (struct jack_data *)ao;
 
 	mpd_jack_stop(jd);
 }
 
+static unsigned
+mpd_jack_delay(struct audio_output *ao)
+{
+	struct jack_data *jd = (struct jack_data *)ao;
+
+	return jd->base.pause && jd->pause && !jd->shutdown
+		? 1000
+		: 0;
+}
+
 static inline jack_default_audio_sample_t
 sample_16_to_jack(int16_t sample)
 {
@@ -648,9 +683,10 @@ mpd_jack_write_samples(struct jack_data *jd, const void *src,
 }
 
 static size_t
-mpd_jack_play(void *data, const void *chunk, size_t size, GError **error_r)
+mpd_jack_play(struct audio_output *ao, const void *chunk, size_t size,
+	      GError **error_r)
 {
-	struct jack_data *jd = data;
+	struct jack_data *jd = (struct jack_data *)ao;
 	const size_t frame_size = audio_format_frame_size(&jd->audio_format);
 	size_t space = 0, space1;
 
@@ -692,19 +728,15 @@ mpd_jack_play(void *data, const void *chunk, size_t size, GError **error_r)
 }
 
 static bool
-mpd_jack_pause(void *data)
+mpd_jack_pause(struct audio_output *ao)
 {
-	struct jack_data *jd = data;
+	struct jack_data *jd = (struct jack_data *)ao;
 
 	if (jd->shutdown)
 		return false;
 
 	jd->pause = true;
 
-	/* due to a MPD API limitation, we have to sleep a little bit
-	   here, to avoid hogging the CPU */
-	g_usleep(50000);
-
 	return true;
 }
 
@@ -716,6 +748,7 @@ const struct audio_output_plugin jack_output_plugin = {
 	.enable = mpd_jack_enable,
 	.disable = mpd_jack_disable,
 	.open = mpd_jack_open,
+	.delay = mpd_jack_delay,
 	.play = mpd_jack_play,
 	.pause = mpd_jack_pause,
 	.close = mpd_jack_close,
