@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2010 The Music Player Daemon Project
+ * Copyright (C) 2003-2011 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -18,6 +18,7 @@
  */
 
 #include "config.h"
+#include "recorder_output_plugin.h"
 #include "output_api.h"
 #include "encoder_plugin.h"
 #include "encoder_list.h"
@@ -34,6 +35,8 @@
 #define G_LOG_DOMAIN "recorder"
 
 struct recorder_output {
+	struct audio_output base;
+
 	/**
 	 * The configured encoder plugin.
 	 */
@@ -64,18 +67,22 @@ recorder_output_quark(void)
 	return g_quark_from_static_string("recorder_output");
 }
 
-static void *
-recorder_output_init(G_GNUC_UNUSED const struct audio_format *audio_format,
-		     const struct config_param *param, GError **error_r)
+static struct audio_output *
+recorder_output_init(const struct config_param *param, GError **error_r)
 {
 	struct recorder_output *recorder = g_new(struct recorder_output, 1);
-	const char *encoder_name;
-	const struct encoder_plugin *encoder_plugin;
+	if (!ao_base_init(&recorder->base, &recorder_output_plugin, param,
+			  error_r)) {
+		g_free(recorder);
+		return NULL;
+	}
 
 	/* read configuration */
 
-	encoder_name = config_get_block_string(param, "encoder", "vorbis");
-	encoder_plugin = encoder_plugin_get(encoder_name);
+	const char *encoder_name =
+		config_get_block_string(param, "encoder", "vorbis");
+	const struct encoder_plugin *encoder_plugin =
+		encoder_plugin_get(encoder_name);
 	if (encoder_plugin == NULL) {
 		g_set_error(error_r, recorder_output_quark(), 0,
 			    "No such encoder: %s", encoder_name);
@@ -95,49 +102,40 @@ recorder_output_init(G_GNUC_UNUSED const struct audio_format *audio_format,
 	if (recorder->encoder == NULL)
 		goto failure;
 
-	return recorder;
+	return &recorder->base;
 
 failure:
+	ao_base_finish(&recorder->base);
 	g_free(recorder);
 	return NULL;
 }
 
 static void
-recorder_output_finish(void *data)
+recorder_output_finish(struct audio_output *ao)
 {
-	struct recorder_output *recorder = data;
+	struct recorder_output *recorder = (struct recorder_output *)ao;
 
 	encoder_finish(recorder->encoder);
+	ao_base_finish(&recorder->base);
 	g_free(recorder);
 }
 
-/**
- * Writes pending data from the encoder to the output file.
- */
 static bool
-recorder_output_encoder_to_file(struct recorder_output *recorder,
-			      GError **error_r)
+recorder_write_to_file(struct recorder_output *recorder,
+		       const void *_data, size_t length,
+		       GError **error_r)
 {
-	size_t size = 0, position, nbytes;
+	assert(length > 0);
 
-	assert(recorder->fd >= 0);
+	const int fd = recorder->fd;
 
-	/* read from the encoder */
+	const uint8_t *data = (const uint8_t *)_data, *end = data + length;
 
-	size = encoder_read(recorder->encoder, recorder->buffer,
-			    sizeof(recorder->buffer));
-	if (size == 0)
-		return true;
-
-	/* write everything into the file */
-
-	position = 0;
 	while (true) {
-		nbytes = write(recorder->fd, recorder->buffer + position,
-			       size - position);
+		ssize_t nbytes = write(fd, data, end - data);
 		if (nbytes > 0) {
-			position += (size_t)nbytes;
-			if (position >= size)
+			data += nbytes;
+			if (data == end)
 				return true;
 		} else if (nbytes == 0) {
 			/* shouldn't happen for files */
@@ -153,12 +151,37 @@ recorder_output_encoder_to_file(struct recorder_output *recorder,
 	}
 }
 
+/**
+ * Writes pending data from the encoder to the output file.
+ */
 static bool
-recorder_output_open(void *data, struct audio_format *audio_format,
+recorder_output_encoder_to_file(struct recorder_output *recorder,
+				GError **error_r)
+{
+	assert(recorder->fd >= 0);
+
+	while (true) {
+		/* read from the encoder */
+
+		size_t size = encoder_read(recorder->encoder, recorder->buffer,
+					   sizeof(recorder->buffer));
+		if (size == 0)
+			return true;
+
+		/* write everything into the file */
+
+		if (!recorder_write_to_file(recorder, recorder->buffer, size,
+					    error_r))
+			return false;
+	}
+}
+
+static bool
+recorder_output_open(struct audio_output *ao,
+		     struct audio_format *audio_format,
 		     GError **error_r)
 {
-	struct recorder_output *recorder = data;
-	bool success;
+	struct recorder_output *recorder = (struct recorder_output *)ao;
 
 	/* create the output file */
 
@@ -174,8 +197,14 @@ recorder_output_open(void *data, struct audio_format *audio_format,
 
 	/* open the encoder */
 
-	success = encoder_open(recorder->encoder, audio_format, error_r);
-	if (!success) {
+	if (!encoder_open(recorder->encoder, audio_format, error_r)) {
+		close(recorder->fd);
+		unlink(recorder->path);
+		return false;
+	}
+
+	if (!recorder_output_encoder_to_file(recorder, error_r)) {
+		encoder_close(recorder->encoder);
 		close(recorder->fd);
 		unlink(recorder->path);
 		return false;
@@ -185,13 +214,13 @@ recorder_output_open(void *data, struct audio_format *audio_format,
 }
 
 static void
-recorder_output_close(void *data)
+recorder_output_close(struct audio_output *ao)
 {
-	struct recorder_output *recorder = data;
+	struct recorder_output *recorder = (struct recorder_output *)ao;
 
 	/* flush the encoder and write the rest to the file */
 
-	if (encoder_flush(recorder->encoder, NULL))
+	if (encoder_end(recorder->encoder, NULL))
 		recorder_output_encoder_to_file(recorder, NULL);
 
 	/* now really close everything */
@@ -202,10 +231,10 @@ recorder_output_close(void *data)
 }
 
 static size_t
-recorder_output_play(void *data, const void *chunk, size_t size,
+recorder_output_play(struct audio_output *ao, const void *chunk, size_t size,
 		     GError **error_r)
 {
-	struct recorder_output *recorder = data;
+	struct recorder_output *recorder = (struct recorder_output *)ao;
 
 	return encoder_write(recorder->encoder, chunk, size, error_r) &&
 		recorder_output_encoder_to_file(recorder, error_r)
