@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2010 The Music Player Daemon Project
+ * Copyright (C) 2003-2011 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -32,13 +32,14 @@
 struct audio_format;
 struct music_buffer;
 struct music_chunk;
+struct player_control;
 
 /**
  * Global initialization: load audio outputs from the configuration
  * file and initialize them.
  */
 void
-audio_output_all_init(void);
+audio_output_all_init(struct player_control *pc);
 
 /**
  * Global finalization: free memory occupied by audio outputs.  All
@@ -127,7 +128,7 @@ audio_output_all_check(void);
  * @return true if there are less than #threshold chunks in the pipe
  */
 bool
-audio_output_all_wait(unsigned threshold);
+audio_output_all_wait(struct player_control *pc, unsigned threshold);
 
 /**
  * Puts all audio outputs into pause mode.  Most implementations will
