@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2010 The Music Player Daemon Project
+ * Copyright (C) 2003-2011 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -17,7 +17,9 @@
  * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
+#include "config.h"
 #include "pcm_buffer.h"
+#include "poison.h"
 
 /**
  * Align the specified size to the next 8k boundary.
@@ -34,12 +36,20 @@ pcm_buffer_get(struct pcm_buffer *buffer, size_t size)
 {
 	assert(buffer != NULL);
 
+	if (size == 0)
+		/* never return NULL, because NULL would be assumed to
+		   be an error condition */
+		size = 1;
+
 	if (buffer->size < size) {
 		/* free the old buffer */
 		g_free(buffer->buffer);
 
 		buffer->size = align_8k(size);
 		buffer->buffer = g_malloc(buffer->size);
+	} else {
+		/* discard old buffer contents */
+		poison_undefined(buffer->buffer, buffer->size);
 	}
 
 	assert(buffer->size >= size);
