@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2010 The Music Player Daemon Project
+ * Copyright (C) 2003-2011 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -20,6 +20,8 @@
 #ifndef PCM_BUFFER_H
 #define PCM_BUFFER_H
 
+#include "check.h"
+
 #include <glib.h>
 
 #include <assert.h>
@@ -63,6 +65,10 @@ pcm_buffer_deinit(struct pcm_buffer *buffer)
 /**
  * Get the buffer, and guarantee a minimum size.  This buffer becomes
  * invalid with the next pcm_buffer_get() call.
+ *
+ * This function will never return NULL, even if size is zero, because
+ * the PCM library uses the NULL return value to signal "error".  An
+ * empty destination buffer is not always an error.
  */
 G_GNUC_MALLOC
 void *
