@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2010 The Music Player Daemon Project
+ * Copyright (C) 2003-2011 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -20,9 +20,10 @@
 #ifndef PCM_MIX_H
 #define PCM_MIX_H
 
-#include <stddef.h>
+#include "audio_format.h"
 
-struct audio_format;
+#include <stdbool.h>
+#include <stddef.h>
 
 /*
  * Linearly mixes two PCM buffers.  Both must have the same length and
@@ -33,13 +34,16 @@ struct audio_format;
  * @param buffer1 the first PCM buffer, and the destination buffer
  * @param buffer2 the second PCM buffer
  * @param size the size of both buffers in bytes
- * @param format the audio format of both buffers
+ * @param format the sample format of both buffers
  * @param portion1 a number between 0.0 and 1.0 specifying the portion
  * of the first buffer in the mix; portion2 = (1.0 - portion1). The value
  * NaN is used by the MixRamp code to specify that simple addition is required.
+ *
+ * @return true on success, false if the format is not supported
  */
-void
+G_GNUC_WARN_UNUSED_RESULT
+bool
 pcm_mix(void *buffer1, const void *buffer2, size_t size,
-	const struct audio_format *format, float portion1);
+	enum sample_format format, float portion1);
 
 #endif
