@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2010 The Music Player Daemon Project
+ * Copyright (C) 2003-2011 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -20,7 +20,7 @@
 #ifndef MPD_PLAYLIST_ANY_H
 #define MPD_PLAYLIST_ANY_H
 
-#include <stdbool.h>
+#include <glib.h>
 
 struct playlist_provider;
 struct input_stream;
@@ -35,6 +35,7 @@ struct input_stream;
  * freed
  */
 struct playlist_provider *
-playlist_open_any(const char *uri, struct input_stream **is_r);
+playlist_open_any(const char *uri, GMutex *mutex, GCond *cond,
+		  struct input_stream **is_r);
 
 #endif
