@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2010 The Music Player Daemon Project
+ * Copyright (C) 2003-2011 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -24,10 +24,13 @@
 #ifndef MPD_PLAYLIST_QUEUE_H
 #define MPD_PLAYLIST_QUEUE_H
 
-#include "playlist.h"
+#include "playlist_error.h"
+
+#include <stdbool.h>
 
 struct playlist_provider;
 struct playlist;
+struct player_control;
 
 /**
  * Loads the contents of a playlist and append it to the specified
@@ -35,17 +38,24 @@ struct playlist;
  *
  * @param uri the URI of the playlist, used to resolve relative song
  * URIs
+ * @param start_index the index of the first song
+ * @param end_index the index of the last song (excluding)
  */
 enum playlist_result
 playlist_load_into_queue(const char *uri, struct playlist_provider *source,
-			 struct playlist *dest);
+			 unsigned start_index, unsigned end_index,
+			 struct playlist *dest, struct player_control *pc,
+			 bool secure);
 
 /**
  * Opens a playlist with a playlist plugin and append to the specified
  * play queue.
  */
 enum playlist_result
-playlist_open_into_queue(const char *uri, struct playlist *dest);
+playlist_open_into_queue(const char *uri,
+			 unsigned start_index, unsigned end_index,
+			 struct playlist *dest, struct player_control *pc,
+			 bool secure);
 
 #endif
 
