@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2010 The Music Player Daemon Project
+ * Copyright (C) 2003-2011 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -23,6 +23,7 @@
 #include "mapper.h"
 #include "song.h"
 #include "uri.h"
+#include "path.h"
 #include "ls.h"
 #include "tag.h"
 
@@ -62,14 +63,19 @@ apply_song_metadata(struct song *dest, const struct song *src)
 		if (path_fs == NULL)
 			return dest;
 
-		tmp = song_file_new(path_fs, NULL);
-		g_free(path_fs);
+		char *path_utf8 = fs_charset_to_utf8(path_fs);
+		if (path_utf8 != NULL)
+			g_free(path_fs);
+		else
+			path_utf8 = path_fs;
+
+		tmp = song_file_new(path_utf8, NULL);
+		g_free(path_utf8);
 
 		merge_song_metadata(tmp, dest, src);
 	} else {
 		tmp = song_file_new(dest->uri, NULL);
 		merge_song_metadata(tmp, dest, src);
-		song_free(dest);
 	}
 
 	if (dest->tag != NULL && dest->tag->time > 0 &&
@@ -80,19 +86,42 @@ apply_song_metadata(struct song *dest, const struct song *src)
 		   (e.g. last track on a CUE file); fix it up here */
 		tmp->tag->time = dest->tag->time - src->start_ms / 1000;
 
+	if (!song_in_database(dest))
+		song_free(dest);
+
 	return tmp;
 }
 
-struct song *
-playlist_check_translate_song(struct song *song, const char *base_uri)
+static struct song *
+playlist_check_load_song(const struct song *song, const char *uri, bool secure)
 {
 	struct song *dest;
 
+	if (uri_has_scheme(uri)) {
+		dest = song_remote_new(uri);
+	} else if (g_path_is_absolute(uri) && secure) {
+		dest = song_file_load(uri, NULL);
+		if (dest == NULL)
+			return NULL;
+	} else {
+		dest = db_get_song(uri);
+		if (dest == NULL)
+			/* not found in database */
+			return NULL;
+	}
+
+	return apply_song_metadata(dest, song);
+}
+
+struct song *
+playlist_check_translate_song(struct song *song, const char *base_uri,
+			      bool secure)
+{
 	if (song_in_database(song))
 		/* already ok */
 		return song;
 
-	char *uri = song->uri;
+	const char *uri = song->uri;
 
 	if (uri_has_scheme(uri)) {
 		if (uri_supported_scheme(uri))
@@ -114,42 +143,27 @@ playlist_check_translate_song(struct song *song, const char *base_uri)
 
 	if (g_path_is_absolute(uri)) {
 		/* XXX fs_charset vs utf8? */
-		char *prefix = map_directory_fs(db_get_root());
+		const char *suffix = map_to_relative_path(uri);
+		assert(suffix != NULL);
 
-		if (prefix == NULL || !g_str_has_prefix(uri, prefix) ||
-		    uri[strlen(prefix)] != '/') {
+		if (suffix != uri)
+			uri = suffix;
+		else if (!secure) {
 			/* local files must be relative to the music
-			   directory */
-			g_free(prefix);
+			   directory when "secure" is enabled */
 			song_free(song);
 			return NULL;
 		}
 
 		base_uri = NULL;
-		uri += strlen(prefix) + 1;
-		g_free(prefix);
 	}
 
+	char *allocated = NULL;
 	if (base_uri != NULL)
-		uri = g_build_filename(base_uri, uri, NULL);
-	else
-		uri = g_strdup(uri);
+		uri = allocated = g_build_filename(base_uri, uri, NULL);
 
-	if (uri_has_scheme(uri)) {
-		dest = song_remote_new(uri);
-		g_free(uri);
-	} else {
-		dest = db_get_song(uri);
-		g_free(uri);
-		if (dest == NULL) {
-			/* not found in database */
-			song_free(song);
-			return dest;
-		}
-	}
-
-	dest = apply_song_metadata(dest, song);
+	struct song *dest = playlist_check_load_song(song, uri, secure);
 	song_free(song);
-
+	g_free(allocated);
 	return dest;
 }
