@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2010 The Music Player Daemon Project
+ * Copyright (C) 2003-2011 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -41,6 +41,10 @@ queue_print_song_info(struct client *client, const struct queue *queue,
 	song_print_info(client, queue_get(queue, position));
 	client_printf(client, "Pos: %u\nId: %u\n",
 		      position, queue_position_to_id(queue, position));
+
+	uint8_t priority = queue_get_priority_at_position(queue, position);
+	if (priority != 0)
+		client_printf(client, "Prio: %u\n", priority);
 }
 
 void
