@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2010 The Music Player Daemon Project
+ * Copyright (C) 2003-2011 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -27,20 +27,16 @@
 #define SONG_BEGIN "song_begin: "
 
 struct song;
-struct songvec;
 struct directory;
 
 void
 song_save(FILE *fp, const struct song *song);
 
-void
-songvec_save(FILE *fp, const struct songvec *sv);
-
 /**
  * Loads a song from the input file.  Reading stops after the
  * "song_end" line.
  *
- * @param error_r location to store the error occuring, or NULL to
+ * @param error_r location to store the error occurring, or NULL to
  * ignore errors
  * @return true on success, false on error
  */
