@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2010 The Music Player Daemon Project
+ * Copyright (C) 2003-2011 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -73,7 +73,7 @@ struct tag_item {
 	 * the value of this tag; this is a variable length string
 	 */
 	char value[sizeof(long)];
-} mpd_packed;
+} gcc_packed;
 
 /**
  * The meta information about a song file.  It is a MPD specific
@@ -88,6 +88,12 @@ struct tag {
 	 */
 	int time;
 
+	/**
+	 * Does this file have an embedded playlist (e.g. embedded CUE
+	 * sheet)?
+	 */
+	bool has_playlist;
+
 	/** an array of tag items */
 	struct tag_item **items;
 
