@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2010 The Music Player Daemon Project
+ * Copyright (C) 2003-2011 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -20,13 +20,21 @@
 #ifndef MPD_TAG_APE_H
 #define MPD_TAG_APE_H
 
+#include "tag_table.h"
+
+#include <stdbool.h>
+
+struct tag_handler;
+
+extern const struct tag_table ape_tags[];
+
 /**
- * Loads the APE tag from a file.
+ * Scan the APE tags of a file.
  *
  * @param path_fs the path of the file in filesystem encoding
- * @return a tag object, or NULL if the file has no APE tag
  */
-struct tag *
-tag_ape_load(const char *path_fs);
+bool
+tag_ape_scan2(const char *path_fs,
+	      const struct tag_handler *handler, void *handler_ctx);
 
 #endif
