@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2010 The Music Player Daemon Project
+ * Copyright (C) 2003-2011 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -19,6 +19,8 @@
 
 #include "config.h"
 #include "tag_id3.h"
+#include "tag_handler.h"
+#include "tag_table.h"
 #include "tag.h"
 #include "riff.h"
 #include "aiff.h"
@@ -126,9 +128,9 @@ import_id3_string(bool is_id3v1, const id3_ucs4_t *ucs4)
  * - string list
  */
 static void
-tag_id3_import_text_frame(struct tag *dest, struct id3_tag *tag,
-			  const struct id3_frame *frame,
-			  enum tag_type type)
+tag_id3_import_text_frame(struct id3_tag *tag, const struct id3_frame *frame,
+			  enum tag_type type,
+			  const struct tag_handler *handler, void *handler_ctx)
 {
 	id3_ucs4_t const *ucs4;
 	id3_utf8_t *utf8;
@@ -164,7 +166,8 @@ tag_id3_import_text_frame(struct tag *dest, struct id3_tag *tag,
 		if (utf8 == NULL)
 			continue;
 
-		tag_add_item(dest, type, (char *)utf8);
+		tag_handler_invoke_tag(handler, handler_ctx,
+				       type, (const char *)utf8);
 		g_free(utf8);
 	}
 }
@@ -174,13 +177,14 @@ tag_id3_import_text_frame(struct tag *dest, struct id3_tag *tag,
  * 4.2).  This is a wrapper for tag_id3_import_text_frame().
  */
 static void
-tag_id3_import_text(struct tag *dest, struct id3_tag *tag, const char *id,
-		    enum tag_type type)
+tag_id3_import_text(struct id3_tag *tag, const char *id, enum tag_type type,
+		    const struct tag_handler *handler, void *handler_ctx)
 {
 	const struct id3_frame *frame;
 	for (unsigned i = 0;
 	     (frame = id3_tag_findframe(tag, id, i)) != NULL; ++i)
-		tag_id3_import_text_frame(dest, tag, frame, type);
+		tag_id3_import_text_frame(tag, frame, type,
+					  handler, handler_ctx);
 }
 
 /**
@@ -193,9 +197,10 @@ tag_id3_import_text(struct tag *dest, struct id3_tag *tag, const char *id,
  * - full string (we use this one)
  */
 static void
-tag_id3_import_comment_frame(struct tag *dest, struct id3_tag *tag,
-			     const struct id3_frame *frame,
-			     enum tag_type type)
+tag_id3_import_comment_frame(struct id3_tag *tag,
+			     const struct id3_frame *frame, enum tag_type type,
+			     const struct tag_handler *handler,
+			     void *handler_ctx)
 {
 	id3_ucs4_t const *ucs4;
 	id3_utf8_t *utf8;
@@ -217,7 +222,7 @@ tag_id3_import_comment_frame(struct tag *dest, struct id3_tag *tag,
 	if (utf8 == NULL)
 		return;
 
-	tag_add_item(dest, type, (char *)utf8);
+	tag_handler_invoke_tag(handler, handler_ctx, type, (const char *)utf8);
 	g_free(utf8);
 }
 
@@ -226,13 +231,14 @@ tag_id3_import_comment_frame(struct tag *dest, struct id3_tag *tag,
  * wrapper for tag_id3_import_comment_frame().
  */
 static void
-tag_id3_import_comment(struct tag *dest, struct id3_tag *tag, const char *id,
-		       enum tag_type type)
+tag_id3_import_comment(struct id3_tag *tag, const char *id, enum tag_type type,
+		       const struct tag_handler *handler, void *handler_ctx)
 {
 	const struct id3_frame *frame;
 	for (unsigned i = 0;
 	     (frame = id3_tag_findframe(tag, id, i)) != NULL; ++i)
-		tag_id3_import_comment_frame(dest, tag, frame, type);
+		tag_id3_import_comment_frame(tag, frame, type,
+					     handler, handler_ctx);
 }
 
 /**
@@ -242,30 +248,26 @@ tag_id3_import_comment(struct tag *dest, struct id3_tag *tag, const char *id,
 static enum tag_type
 tag_id3_parse_txxx_name(const char *name)
 {
-	static const struct {
-		enum tag_type type;
-		const char *name;
-	} musicbrainz_txxx[] = {
-		{ TAG_ALBUM_ARTIST_SORT, "ALBUMARTISTSORT" },
-		{ TAG_MUSICBRAINZ_ARTISTID, "MusicBrainz Artist Id" },
-		{ TAG_MUSICBRAINZ_ALBUMID, "MusicBrainz Album Id" },
-		{ TAG_MUSICBRAINZ_ALBUMARTISTID,
-		  "MusicBrainz Album Artist Id" },
-		{ TAG_MUSICBRAINZ_TRACKID, "MusicBrainz Track Id" },
+	static const struct tag_table txxx_tags[] = {
+		{ "ALBUMARTISTSORT", TAG_ALBUM_ARTIST_SORT },
+		{ "MusicBrainz Artist Id", TAG_MUSICBRAINZ_ARTISTID },
+		{ "MusicBrainz Album Id", TAG_MUSICBRAINZ_ALBUMID },
+		{ "MusicBrainz Album Artist Id",
+		  TAG_MUSICBRAINZ_ALBUMARTISTID },
+		{ "MusicBrainz Track Id", TAG_MUSICBRAINZ_TRACKID },
+		{ NULL, TAG_NUM_OF_ITEM_TYPES }
 	};
 
-	for (unsigned i = 0; i < G_N_ELEMENTS(musicbrainz_txxx); ++i)
-		if (strcmp(name, musicbrainz_txxx[i].name) == 0)
-			return musicbrainz_txxx[i].type;
-
-	return TAG_NUM_OF_ITEM_TYPES;
+	return tag_table_lookup(txxx_tags, name);
 }
 
 /**
  * Import all known MusicBrainz tags from TXXX frames.
  */
 static void
-tag_id3_import_musicbrainz(struct tag *mpd_tag, struct id3_tag *id3_tag)
+tag_id3_import_musicbrainz(struct id3_tag *id3_tag,
+			   const struct tag_handler *handler,
+			   void *handler_ctx)
 {
 	for (unsigned i = 0;; ++i) {
 		const struct id3_frame *frame;
@@ -280,17 +282,21 @@ tag_id3_import_musicbrainz(struct tag *mpd_tag, struct id3_tag *id3_tag)
 		if (name == NULL)
 			continue;
 
-		type = tag_id3_parse_txxx_name((const char*)name);
-		free(name);
-
-		if (type == TAG_NUM_OF_ITEM_TYPES)
-			continue;
-
 		value = tag_id3_getstring(frame, 2);
 		if (value == NULL)
 			continue;
 
-		tag_add_item(mpd_tag, type, (const char*)value);
+		tag_handler_invoke_pair(handler, handler_ctx,
+					(const char *)name,
+					(const char *)value);
+
+		type = tag_id3_parse_txxx_name((const char*)name);
+		free(name);
+
+		if (type != TAG_NUM_OF_ITEM_TYPES)
+			tag_handler_invoke_tag(handler, handler_ctx,
+					       type, (const char*)value);
+
 		free(value);
 	}
 }
@@ -299,7 +305,8 @@ tag_id3_import_musicbrainz(struct tag *mpd_tag, struct id3_tag *id3_tag)
  * Imports the MusicBrainz TrackId from the UFID tag.
  */
 static void
-tag_id3_import_ufid(struct tag *mpd_tag, struct id3_tag *id3_tag)
+tag_id3_import_ufid(struct id3_tag *id3_tag,
+		    const struct tag_handler *handler, void *handler_ctx)
 {
 	for (unsigned i = 0;; ++i) {
 		const struct id3_frame *frame;
@@ -329,35 +336,54 @@ tag_id3_import_ufid(struct tag *mpd_tag, struct id3_tag *id3_tag)
 		if (value == NULL || length == 0)
 			continue;
 
-		tag_add_item_n(mpd_tag, TAG_MUSICBRAINZ_TRACKID,
-			       (const char*)value, length);
+		char *p = g_strndup((const char *)value, length);
+		tag_handler_invoke_tag(handler, handler_ctx,
+				       TAG_MUSICBRAINZ_TRACKID, p);
+		g_free(p);
 	}
 }
 
+static void
+scan_id3_tag(struct id3_tag *tag,
+	     const struct tag_handler *handler, void *handler_ctx)
+{
+	tag_id3_import_text(tag, ID3_FRAME_ARTIST, TAG_ARTIST,
+			    handler, handler_ctx);
+	tag_id3_import_text(tag, ID3_FRAME_ALBUM_ARTIST,
+			    TAG_ALBUM_ARTIST, handler, handler_ctx);
+	tag_id3_import_text(tag, ID3_FRAME_ARTIST_SORT,
+			    TAG_ARTIST_SORT, handler, handler_ctx);
+	tag_id3_import_text(tag, ID3_FRAME_ALBUM_ARTIST_SORT,
+			    TAG_ALBUM_ARTIST_SORT, handler, handler_ctx);
+	tag_id3_import_text(tag, ID3_FRAME_TITLE, TAG_TITLE,
+			    handler, handler_ctx);
+	tag_id3_import_text(tag, ID3_FRAME_ALBUM, TAG_ALBUM,
+			    handler, handler_ctx);
+	tag_id3_import_text(tag, ID3_FRAME_TRACK, TAG_TRACK,
+			    handler, handler_ctx);
+	tag_id3_import_text(tag, ID3_FRAME_YEAR, TAG_DATE,
+			    handler, handler_ctx);
+	tag_id3_import_text(tag, ID3_FRAME_GENRE, TAG_GENRE,
+			    handler, handler_ctx);
+	tag_id3_import_text(tag, ID3_FRAME_COMPOSER, TAG_COMPOSER,
+			    handler, handler_ctx);
+	tag_id3_import_text(tag, "TPE3", TAG_PERFORMER,
+			    handler, handler_ctx);
+	tag_id3_import_text(tag, "TPE4", TAG_PERFORMER, handler, handler_ctx);
+	tag_id3_import_comment(tag, ID3_FRAME_COMMENT, TAG_COMMENT,
+			       handler, handler_ctx);
+	tag_id3_import_text(tag, ID3_FRAME_DISC, TAG_DISC,
+			    handler, handler_ctx);
+
+	tag_id3_import_musicbrainz(tag, handler, handler_ctx);
+	tag_id3_import_ufid(tag, handler, handler_ctx);
+}
+
 struct tag *tag_id3_import(struct id3_tag * tag)
 {
 	struct tag *ret = tag_new();
 
-	tag_id3_import_text(ret, tag, ID3_FRAME_ARTIST, TAG_ARTIST);
-	tag_id3_import_text(ret, tag, ID3_FRAME_ALBUM_ARTIST,
-			    TAG_ALBUM_ARTIST);
-	tag_id3_import_text(ret, tag, ID3_FRAME_ARTIST_SORT,
-			    TAG_ARTIST_SORT);
-	tag_id3_import_text(ret, tag, ID3_FRAME_ALBUM_ARTIST_SORT,
-			    TAG_ALBUM_ARTIST_SORT);
-	tag_id3_import_text(ret, tag, ID3_FRAME_TITLE, TAG_TITLE);
-	tag_id3_import_text(ret, tag, ID3_FRAME_ALBUM, TAG_ALBUM);
-	tag_id3_import_text(ret, tag, ID3_FRAME_TRACK, TAG_TRACK);
-	tag_id3_import_text(ret, tag, ID3_FRAME_YEAR, TAG_DATE);
-	tag_id3_import_text(ret, tag, ID3_FRAME_GENRE, TAG_GENRE);
-	tag_id3_import_text(ret, tag, ID3_FRAME_COMPOSER, TAG_COMPOSER);
-	tag_id3_import_text(ret, tag, "TPE3", TAG_PERFORMER);
-	tag_id3_import_text(ret, tag, "TPE4", TAG_PERFORMER);
-	tag_id3_import_comment(ret, tag, ID3_FRAME_COMMENT, TAG_COMMENT);
-	tag_id3_import_text(ret, tag, ID3_FRAME_DISC, TAG_DISC);
-
-	tag_id3_import_musicbrainz(ret, tag);
-	tag_id3_import_ufid(ret, tag);
+	scan_id3_tag(tag, &add_tag_handler, ret);
 
 	if (tag_is_empty(ret)) {
 		tag_free(ret);
@@ -515,30 +541,44 @@ tag_id3_riff_aiff_load(FILE *file)
 	return tag;
 }
 
-struct tag *tag_id3_load(const char *file)
+struct id3_tag *
+tag_id3_load(const char *path_fs, GError **error_r)
 {
-	struct tag *ret;
-	struct id3_tag *tag;
-	FILE *stream;
-
-	stream = fopen(file, "rb");
-	if (!stream) {
-		g_debug("tag_id3_load: Failed to open file: '%s', %s",
-			file, strerror(errno));
+	FILE *file = fopen(path_fs, "rb");
+	if (file == NULL) {
+		g_set_error(error_r, g_file_error_quark(), errno,
+			    "Failed to open file %s: %s",
+			    path_fs, g_strerror(errno));
 		return NULL;
 	}
 
-	tag = tag_id3_find_from_beginning(stream);
-	if (tag == NULL)
-		tag = tag_id3_riff_aiff_load(stream);
-	if (!tag)
-		tag = tag_id3_find_from_end(stream);
+	struct id3_tag *tag = tag_id3_find_from_beginning(file);
+	if (tag == NULL) {
+		tag = tag_id3_riff_aiff_load(file);
+		if (tag == NULL)
+			tag = tag_id3_find_from_end(file);
+	}
 
-	fclose(stream);
+	fclose(file);
+	return tag;
+}
 
-	if (!tag)
-		return NULL;
-	ret = tag_id3_import(tag);
+bool
+tag_id3_scan(const char *path_fs,
+	     const struct tag_handler *handler, void *handler_ctx)
+{
+	GError *error = NULL;
+	struct id3_tag *tag = tag_id3_load(path_fs, &error);
+	if (tag == NULL) {
+		if (error != NULL) {
+			g_warning("%s", error->message);
+			g_error_free(error);
+		}
+
+		return false;
+	}
+
+	scan_id3_tag(tag, handler, handler_ctx);
 	id3_tag_delete(tag);
-	return ret;
+	return true;
 }
