@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2009 The Music Player Daemon Project
+ * Copyright (C) 2003-2011 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -20,24 +20,42 @@
 #ifndef MPD_TAG_ID3_H
 #define MPD_TAG_ID3_H
 
-#include "config.h"
-
-struct tag;
-
-#ifdef HAVE_ID3TAG
-struct id3_tag;
-struct tag *tag_id3_import(struct id3_tag *);
-
-struct tag *tag_id3_load(const char *file);
-
-#else
+#include "check.h"
 
 #include <glib.h>
 
-static inline struct tag *
-tag_id3_load(G_GNUC_UNUSED const char *file)
+#include <stdbool.h>
+
+struct tag_handler;
+struct tag;
+
+#ifdef HAVE_ID3TAG
+
+bool
+tag_id3_scan(const char *path_fs,
+	     const struct tag_handler *handler, void *handler_ctx);
+
+struct id3_tag;
+struct tag *tag_id3_import(struct id3_tag *);
+
+/**
+ * Loads the ID3 tags from the file into a libid3tag object.  The
+ * return value must be freed with id3_tag_delete().
+ *
+ * @return NULL on error or if no ID3 tag was found in the file (no
+ * GError will be set)
+ */
+struct id3_tag *
+tag_id3_load(const char *path_fs, GError **error_r);
+
+#else
+
+static inline bool
+tag_id3_scan(G_GNUC_UNUSED const char *path_fs,
+	     G_GNUC_UNUSED const struct tag_handler *handler,
+	     G_GNUC_UNUSED void *handler_ctx)
 {
-	return NULL;
+	return false;
 }
 
 #endif
