@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2010 The Music Player Daemon Project
+ * Copyright (C) 2003-2011 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -24,18 +24,40 @@
 
 #include <glib.h>
 
+struct tag_table {
+	const char *name;
+
+	enum tag_type type;
+};
+
+/**
+ * Looks up a string in a tag translation table (case sensitive).
+ * Returns TAG_NUM_OF_ITEM_TYPES if the specified name was not found
+ * in the table.
+ */
+G_GNUC_PURE
+static inline enum tag_type
+tag_table_lookup(const struct tag_table *table, const char *name)
+{
+	for (; table->name != NULL; ++table)
+		if (strcmp(name, table->name) == 0)
+			return table->type;
+
+	return TAG_NUM_OF_ITEM_TYPES;
+}
+
 /**
  * Looks up a string in a tag translation table (case insensitive).
  * Returns TAG_NUM_OF_ITEM_TYPES if the specified name was not found
  * in the table.
  */
+G_GNUC_PURE
 static inline enum tag_type
-tag_table_lookup(const char *const* table, const char *name)
+tag_table_lookup_i(const struct tag_table *table, const char *name)
 {
-	for (unsigned i = 0; i < TAG_NUM_OF_ITEM_TYPES; i++)
-		if (table[i] != NULL &&
-		    g_ascii_strcasecmp(name, table[i]) == 0)
-			return (enum tag_type)i;
+	for (; table->name != NULL; ++table)
+		if (g_ascii_strcasecmp(name, table->name) == 0)
+			return table->type;
 
 	return TAG_NUM_OF_ITEM_TYPES;
 }
