@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2010 The Music Player Daemon Project
+ * Copyright (C) 2003-2011 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -20,61 +20,54 @@
 #include "config.h"
 #include "timer.h"
 #include "audio_format.h"
+#include "clock.h"
 
 #include <glib.h>
 
 #include <assert.h>
 #include <limits.h>
-#include <sys/time.h>
 #include <stddef.h>
 
-static uint64_t now(void)
+struct timer *timer_new(const struct audio_format *af)
 {
-	struct timeval tv;
-
-	gettimeofday(&tv, NULL);
-
-	return ((uint64_t)tv.tv_sec * 1000000) + tv.tv_usec;
-}
-
-Timer *timer_new(const struct audio_format *af)
-{
-	Timer *timer = g_new(Timer, 1);
-	timer->time = 0;
-	timer->started = 0;
-	timer->rate = af->sample_rate * audio_format_frame_size(af);
+	struct timer *timer = g_new(struct timer, 1);
+	timer->time = 0; // us
+	timer->started = 0; // false
+	timer->rate = af->sample_rate * audio_format_frame_size(af); // samples per second
 
 	return timer;
 }
 
-void timer_free(Timer *timer)
+void timer_free(struct timer *timer)
 {
 	g_free(timer);
 }
 
-void timer_start(Timer *timer)
+void timer_start(struct timer *timer)
 {
-	timer->time = now();
+	timer->time = monotonic_clock_us();
 	timer->started = 1;
 }
 
-void timer_reset(Timer *timer)
+void timer_reset(struct timer *timer)
 {
 	timer->time = 0;
 	timer->started = 0;
 }
 
-void timer_add(Timer *timer, int size)
+void timer_add(struct timer *timer, int size)
 {
 	assert(timer->started);
 
+	// (size samples) / (rate samples per second) = duration seconds
+	// duration seconds * 1000000 = duration us
 	timer->time += ((uint64_t)size * 1000000) / timer->rate;
 }
 
 unsigned
-timer_delay(const Timer *timer)
+timer_delay(const struct timer *timer)
 {
-	int64_t delay = (int64_t)(timer->time - now()) / 1000;
+	int64_t delay = (int64_t)(timer->time - monotonic_clock_us()) / 1000;
 	if (delay < 0)
 		return 0;
 
@@ -84,13 +77,13 @@ timer_delay(const Timer *timer)
 	return delay;
 }
 
-void timer_sync(Timer *timer)
+void timer_sync(struct timer *timer)
 {
 	int64_t sleep_duration;
 
 	assert(timer->started);
 
-	sleep_duration = timer->time - now();
+	sleep_duration = timer->time - monotonic_clock_us();
 	if (sleep_duration > 0)
 		g_usleep(sleep_duration);
 }
