@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2010 The Music Player Daemon Project
+ * Copyright (C) 2003-2011 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -24,28 +24,28 @@
 
 struct audio_format;
 
-typedef struct _Timer {
+struct timer {
 	uint64_t time;
 	int started;
 	int rate;
-} Timer;
+};
 
-Timer *timer_new(const struct audio_format *af);
+struct timer *timer_new(const struct audio_format *af);
 
-void timer_free(Timer *timer);
+void timer_free(struct timer *timer);
 
-void timer_start(Timer *timer);
+void timer_start(struct timer *timer);
 
-void timer_reset(Timer *timer);
+void timer_reset(struct timer *timer);
 
-void timer_add(Timer *timer, int size);
+void timer_add(struct timer *timer, int size);
 
 /**
  * Returns the number of milliseconds to sleep to get back to sync.
  */
 unsigned
-timer_delay(const Timer *timer);
+timer_delay(const struct timer *timer);
 
-void timer_sync(Timer *timer);
+void timer_sync(struct timer *timer);
 
 #endif
