@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2010 The Music Player Daemon Project
+ * Copyright (C) 2003-2011 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -18,7 +18,10 @@
  */
 
 #include "config.h"
-#include "update_internal.h"
+#include "update.h"
+#include "update_queue.h"
+#include "update_walk.h"
+#include "update_remove.h"
 #include "update.h"
 #include "database.h"
 #include "mapper.h"
@@ -34,6 +37,9 @@
 
 #include <assert.h>
 
+#undef G_LOG_DOMAIN
+#define G_LOG_DOMAIN "update"
+
 static enum update_progress {
 	UPDATE_PROGRESS_IDLE = 0,
 	UPDATE_PROGRESS_RUNNING = 1,
@@ -68,8 +74,14 @@ static void * update_task(void *_path)
 
 	modified = update_walk(path, discard);
 
-	if (modified || !db_exists())
-		db_save();
+	if (modified || !db_exists()) {
+		GError *error = NULL;
+		if (!db_save(&error)) {
+			g_warning("Failed to save database: %s",
+				  error->message);
+			g_error_free(error);
+		}
+	}
 
 	if (path != NULL && *path != 0)
 		g_debug("finished: %s", path);
