@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2010 The Music Player Daemon Project
+ * Copyright (C) 2003-2011 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -34,13 +34,13 @@ bool uri_has_scheme(const char *uri)
 const char *
 uri_get_suffix(const char *uri)
 {
-	const char *suffix = strrchr(g_basename(uri), '.');
+	const char *suffix = strrchr(uri, '.');
 	if (suffix == NULL)
 		return NULL;
 
 	++suffix;
 
-	if (strchr(suffix, '/') != NULL)
+	if (strpbrk(suffix, "/\\") != NULL)
 		return NULL;
 
 	return suffix;
