@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2009 The Music Player Daemon Project
+ * Copyright (C) 2003-2011 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -20,6 +20,9 @@
 #ifndef MPD_UTILS_H
 #define MPD_UTILS_H
 
+#include <glib.h>
+#include <stdbool.h>
+
 #ifndef assert_static
 /* Compile time assertion developed by Ralf Holly */
 /* http://pera-software.com/articles/compile-time-assertions.pdf */
@@ -29,10 +32,7 @@
 	} while (0)
 #endif /* !assert_static */
 
-char *parsePath(char *path);
-
-int set_nonblocking(int fd);
-
-int stringFoundInStringArray(const char *const*array, const char *suffix);
+char *
+parsePath(const char *path, GError **error_r);
 
 #endif
