@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2010 The Music Player Daemon Project
+ * Copyright (C) 2003-2011 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -33,6 +33,16 @@
 #ifdef HAVE_PULSE
 #include "output/pulse_output_plugin.h"
 
+void
+pulse_output_lock(G_GNUC_UNUSED struct pulse_output *po)
+{
+}
+
+void
+pulse_output_unlock(G_GNUC_UNUSED struct pulse_output *po)
+{
+}
+
 void
 pulse_output_set_mixer(G_GNUC_UNUSED struct pulse_output *po,
 		       G_GNUC_UNUSED struct pulse_mixer *pm)
@@ -55,6 +65,24 @@ pulse_output_set_volume(G_GNUC_UNUSED struct pulse_output *po,
 
 #endif
 
+#ifdef HAVE_ROAR
+#include "output/roar_output_plugin.h"
+
+int
+roar_output_get_volume(G_GNUC_UNUSED struct roar *roar)
+{
+	return -1;
+}
+
+bool
+roar_output_set_volume(G_GNUC_UNUSED struct roar *roar,
+		       G_GNUC_UNUSED unsigned volume)
+{
+	return true;
+}
+
+#endif
+
 void
 event_pipe_emit(G_GNUC_UNUSED enum pipe_event event)
 {
@@ -68,8 +96,8 @@ filter_plugin_by_name(G_GNUC_UNUSED const char *name)
 }
 
 bool
-pcm_volume(G_GNUC_UNUSED void *buffer, G_GNUC_UNUSED int length,
-	   G_GNUC_UNUSED const struct audio_format *format,
+pcm_volume(G_GNUC_UNUSED void *buffer, G_GNUC_UNUSED size_t length,
+	   G_GNUC_UNUSED enum sample_format format,
 	   G_GNUC_UNUSED int volume)
 {
 	assert(false);
