@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2010 The Music Player Daemon Project
+ * Copyright (C) 2003-2011 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -183,7 +183,7 @@ int main(int argc, char **argv)
 
 		nbytes = write(1, dest, length);
 		if (nbytes < 0) {
-			g_printerr("Failed to write: %s\n", strerror(errno));
+			g_printerr("Failed to write: %s\n", g_strerror(errno));
 			filter_close(filter);
 			filter_free(filter);
 			return 1;
